from answer_views import *
from background_views import *
from background_category_views import *
from candidate_views import *
from category_views import *
from election_views import *
from medianaranja_views import *
from personal_data_views import *
from question_views import *